#pragma once
#ifndef VECTOR_H_
#define VECTOR_H_

#include <math.h>

namespace EPOC
{
	class Vector2D
	{
	public:
		Vector2D() : m_fX(0.0f), m_fY(0.0f){};
		Vector2D(float fX, float fY) : m_fX(fX), m_fY(fY){};

		virtual float Length()				{ return sqrtf( ( m_fX * m_fX ) + ( m_fY * m_fY )); }
		virtual float LengthSq()			{ return ( m_fX * m_fX ) + ( m_fY * m_fY ); }
		virtual void Normalize()			{ float fLength = Length(); m_fX /= fLength; m_fY /= fLength;}
		virtual Vector2D NormalizeCopy()	{ Vector2D vNormalized; float fLength = Length(); vNormalized.m_fX = m_fX / fLength; vNormalized.m_fY = m_fY / fLength; return vNormalized; }

		virtual float DotProduct(Vector2D *pVector)	{ return m_fX * pVector->m_fX + m_fY * pVector->m_fY; }
		virtual Vector2D Perpendicular()				{ Vector2D vPerpendicular; vPerpendicular.m_fX = -m_fY; vPerpendicular.m_fY = m_fX; return vPerpendicular; }
		
		const Vector2D Vector2D::operator+(const Vector2D &other) const 
		{
			Vector2D result = *this;  // Make a copy of myself.  Same as MyClass result(*this);
			//result += other;            // Use += to add other to the copy.
			result.m_fX += other.m_fX;
			result.m_fY += other.m_fY;
		
			return result;              // All done!
		}

		const Vector2D Vector2D::operator-(const Vector2D &other) const 
		{
			Vector2D result = *this;  // Make a copy of myself.  Same as MyClass result(*this);
			//result += other;            // Use += to add other to the copy.
			result.m_fX -= other.m_fX;
			result.m_fY -= other.m_fY;
		
			return result;              // All done!
		}

		const Vector2D Vector2D::operator*(const float &other) const 
		{
			Vector2D result = *this;  // Make a copy of myself.  Same as MyClass result(*this);
			//result += other;            // Use += to add other to the copy.
			result.m_fX *= other;
			result.m_fY *= other;
		
			return result;              // All done!
		}

		const Vector2D Vector2D::operator/(const float &other) const 
		{
			Vector2D result = *this;  // Make a copy of myself.  Same as MyClass result(*this);
			//result += other;            // Use += to add other to the copy.
			result.m_fX /= other;
			result.m_fY /= other;
		
			return result;              // All done!
		}

		Vector2D& Vector2D::operator+=(const Vector2D& right)
		{
			*this = *this + right;
			return *this;
		}

		Vector2D& Vector2D::operator-=(const Vector2D& right)
		{
			*this = *this - right;
			return *this;
		}

		Vector2D& Vector2D::operator*=(const float& right)
		{
			*this = *this * right;
			return *this;
		}

		Vector2D& Vector2D::operator/=(const float& right)
		{
			*this = *this / right;
			return *this;
		}

		bool Vector2D::operator==(const Vector2D &other) const 
		{
			return (this->m_fX == other.m_fX && this->m_fY == other.m_fY);
		}

		bool Vector2D::operator!=(const Vector2D &other) const 
		{
			return !(*this == other);
		}

	public:
		float m_fX;
		float m_fY;
	
	}; // class Vector2D
}; // namespace EPOC

#endif